//Michael Klements
//The DIY Life
//Arc Reactor CPU Monitor
//25 May 2020

#include <SPI.h>                //Include the libraries for the display
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128        //OLED display width, in pixels
#define SCREEN_HEIGHT 32        //OLED display height, in pixels

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);   //Create the display object

int pinReactor = 5;             //Pin to reactor
int brightness = 25;            //How bright the reactor LED is at minimum. Its distracting if it turns off entirely.
int fadeAmount = 1;             //How many increments to fade the reactor LED by in each loop cycle

char receivedChars[5];          //Variables to track the incoming Serial data
boolean newData = false;

int cpuLoad = 0;                //Variable for the CPU load

void setup() 
{ 
  Serial.begin(9600);                                     //Begin Serial communication with the PC
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C))          // Address 0x3C for 128x32 display
  {
    Serial.println(F("SSD1306 allocation failed"));
    for(;;);                                              //Don't proceed if it can't communicate with the display
  }
  display.clearDisplay();                                 //Clear the display
  display.setTextSize(1);                                 //Set the display text size
  display.setTextColor(SSD1306_WHITE);                    //Draw white text
  updateDisplay();                                        //Update the displayed CPU load, 0% initially
}

void loop() 
{ 
  analogWrite(pinReactor, brightness);                    //Set the reactor LED brightness
  brightness = brightness + fadeAmount;                   //Increment the brightness by the fade amount
  if (brightness <= 25 || brightness >= 255)              //Reverse the direction of the fade increment once fully bright or dim
  {
    fadeAmount = -fadeAmount;
  }
  if (Serial.available())                                 //If an updated CPU load is received
  {
    delay(10);
    static byte ndx = 0;                                  //Index to keep track of received char position
    char endMarker = '%';                                 //Symbol indicating that the full CPU load value has been recieved
    char rc;                                              //Temporary variable to store the received char
    cpuLoad = 0;                                          //Reset CPU load variable to 0
    while (Serial.available() > 0 && newData == false)    //While data is being recieved
    {
      rc = Serial.read();                                 //Save data to rc
      if (rc != endMarker)                                //Check that the end of the data symbol hasn't been received
      {
            receivedChars[ndx] = rc;                      //Save char received to receivedChars array
            int temp = (int)rc - 48;                      //Convert ASCII char to int
            if(temp >= 0)                                 //Check that the converted char was not a space (ASCII 32)
            {
              if(ndx == 0)                                //Update CPU load based on 100s, 10s or units position
                cpuLoad = cpuLoad + temp*100;
              else if(ndx == 1)
                cpuLoad = cpuLoad + temp*10;
              else
                cpuLoad = cpuLoad + temp;
            }
            ndx++;                                        //Increment the index
        }
        else 
        {
            receivedChars[ndx] = '\0';                    //Terminate the string
            ndx = 0;                                      //Reset the index
            newData = true;                               //Terminate the loop
        }
    }
    if (newData == true)                                  //Reset new data
    {
        newData = false;
    }
    updateDisplay();                                      //Update the displayed CPU load
  }
  int delayDuration = map(cpuLoad, 0, 100, 10000, 500);   //Maps CPU Load to a delay duration, higher load is a shorter fade time
  delayMicroseconds(delayDuration);                       //Wait the calculated delay duration
}

void updateDisplay ()                                     //Update the displayed text
{
  display.clearDisplay();
  display.setCursor(40,5);
  display.println(F("CPU Load"));
  display.setCursor(55,15);
  display.print(cpuLoad);
  display.print(F("%"));
  display.display();
}
